package cz.uhk.fim.smarttodo.common;

import cz.uhk.fim.smarttodo.common.validation.StValidationException;
import cz.uhk.fim.smarttodo.common.validation.ValidationInfo;
import cz.uhk.fim.smarttodo.common.validation.ValidationMessage;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.ApplicationContext;
import org.springframework.context.NoSuchMessageException;
import org.springframework.context.i18n.LocaleContextHolder;
import org.springframework.stereotype.Service;

import javax.validation.ConstraintViolation;
import javax.validation.ConstraintViolationException;
import javax.validation.ValidationException;
import java.util.Locale;

@Service
public class StCommonService {

    private final ApplicationContext applicationContext;

    @Autowired
    public StCommonService(ApplicationContext applicationContext) {
        this.applicationContext = applicationContext;
    }

    public ValidationInfo convertValidationException(ValidationException ex) {

        ValidationInfo validationInfo = new ValidationInfo();

        if (ex instanceof ConstraintViolationException) {
            ConstraintViolationException cve = (ConstraintViolationException) ex;
            for (ConstraintViolation<?> cv : cve.getConstraintViolations()) {
                validationInfo.getMessages().add(new ValidationMessage(cv));
            }

        } else if (ex instanceof StValidationException) {
            validationInfo.setMessages(((StValidationException) ex).getMessages());

            for (ValidationMessage vm : validationInfo.getMessages()) {
                vm.resolveMessage(this, LocaleContextHolder.getLocale());
            }
        } else {
            validationInfo.getMessages().add(new ValidationMessage(ex.getMessage()));
        }


        return validationInfo;
    }

    public String format(String code, Locale locale, Object... args) {
        try {
            return applicationContext.getMessage(code, args, locale);
        } catch (NoSuchMessageException e) {
            return code;
        }
    }
}
