package cz.uhk.fim.smarttodo.common.validation;

import com.fasterxml.jackson.annotation.JsonIgnore;
import com.fasterxml.jackson.annotation.JsonProperty;
import cz.uhk.fim.smarttodo.common.StCommonService;
import org.apache.commons.lang3.StringUtils;

import javax.validation.ConstraintViolation;

import java.util.Locale;
import java.util.Map;


public class ValidationMessage {
    private String propertyPath;

    private String message;

    private String errorCode;

    private Object[] params;

    private Map<String, Object> clientParams;


    public ValidationMessage() {
    }

    public ValidationMessage(String message) {
        this.message = message;
    }


    public ValidationMessage(String propertyPath, String message) {
        this.propertyPath = propertyPath;
        this.message = message;
    }

    public ValidationMessage(String errorCode, Object[] params) {
        this.errorCode = errorCode;
        this.params = params;
    }

    public ValidationMessage(String errorCode, Object[] params, String propertyPath) {
        this.errorCode = errorCode;

        this.params = params;
        this.propertyPath = propertyPath;
    }

    public ValidationMessage(String propertyPath, String errorCode, Object[] params) {
        this.propertyPath = propertyPath;
        this.errorCode = errorCode;
        this.params = params;
    }


    public ValidationMessage(ConstraintViolation<?> cv) {
        this(cv.getMessage());

        propertyPath = cv.getPropertyPath().toString();
    }

    public ValidationMessage resolveMessage(StCommonService stCommonService, Locale locale) {
        if (StringUtils.isNotBlank(getErrorCode())) {
            setMessage(stCommonService.format(getErrorCode(), locale, getParams()));
            setErrorCode(null);
        }
        return this;
    }

    @JsonProperty
    public String getMessage() {
        return message;
    }

    public void setMessage(String message) {
        this.message = message;
    }

    public String getPropertyPath() {
        return propertyPath;
    }


    @Override
    public String toString() {
        return getMessage();
    }


    public void setPropertyPath(String propertyPath) {
        this.propertyPath = propertyPath;
    }

    @JsonIgnore
    public Object[] getParams() {
        return params;
    }

    public void setParams(Object[] params) {
        this.params = params;
    }

    public String getErrorCode() {
        return errorCode;
    }

    public void setErrorCode(String errorCode) {
        this.errorCode = errorCode;
    }


    public Map<String, Object> getClientParams() {
        return clientParams;
    }

    public void setClientParams(Map<String, Object> clientParams) {
        this.clientParams = clientParams;
    }
}
