package cz.uhk.fim.smarttodo.controller;


import cz.uhk.fim.smarttodo.common.AppControllerAncestor;
import cz.uhk.fim.smarttodo.domain.Event;
import cz.uhk.fim.smarttodo.domain.User;
import cz.uhk.fim.smarttodo.dto.event.EventDto;
import cz.uhk.fim.smarttodo.service.EventService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.core.Authentication;
import org.springframework.web.bind.annotation.*;

import java.util.List;
import java.util.Locale;

@RestController
@RequestMapping(value = "/event")
public class EventController extends AppControllerAncestor {

    private final EventService eventService;

    @Autowired
    public EventController(EventService eventService) {
        this.eventService = eventService;
    }

    @GetMapping(value = "/all")
    public @ResponseBody
    List<EventDto> readAllUsersEvents(Authentication authentication, Locale locale) {
        final User user = getUser(authentication, locale);

        return eventService.getAllUsersEvents(user);
    }

    @PostMapping(value = "/create")
    public @ResponseBody
    EventDto createNewEvent(@RequestBody EventDto dto, Authentication authentication, Locale locale) {
        final User user = getUser(authentication, locale);

        Event newEvent = eventService.createNewEvent(user, dto);

        return newEvent.mapToEventDto();
    }

    @PostMapping(value = "/delete/{eventId}")
    public @ResponseBody
    void deleteEvent(@PathVariable Integer eventId, Authentication authentication, Locale locale) {
        final User user = getUser(authentication, locale);

        eventService.deleteEvent(user, eventId);
    }

    @PostMapping(value = "/update")
    public @ResponseBody
    EventDto updateEvent(@RequestBody EventDto dto, Authentication authentication, Locale locale) {
        final User user = getUser(authentication, locale);

        Event event = eventService.updateEvent(user, dto);

        return event.mapToEventDto();
    }

}
