package cz.uhk.fim.smarttodo.controller;

import cz.uhk.fim.smarttodo.common.AppControllerAncestor;
import cz.uhk.fim.smarttodo.domain.TodoItem;
import cz.uhk.fim.smarttodo.domain.User;
import cz.uhk.fim.smarttodo.dto.NewTodoListRequestDto;
import cz.uhk.fim.smarttodo.dto.TodoItemResponseDto;
import cz.uhk.fim.smarttodo.dto.TodoListResponseDto;
import cz.uhk.fim.smarttodo.dto.todo.TodoItemDto;
import cz.uhk.fim.smarttodo.dto.user.UserForListEditDto;
import cz.uhk.fim.smarttodo.service.TodoService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.core.Authentication;
import org.springframework.web.bind.annotation.*;

import java.util.List;
import java.util.Locale;
import java.util.stream.Collectors;

@RestController
@RequestMapping(value = "/todo")
public class TodoController extends AppControllerAncestor {

    private final TodoService todoService;

    @Autowired
    public TodoController(TodoService todoService) {
        this.todoService = todoService;
    }


    @GetMapping(value = "/list/all")
    public @ResponseBody
    List<TodoListResponseDto> readAllUsersTodoLists(Authentication authentication, Locale locale) {
        final User user = getUser(authentication, locale);

        return todoService.getAllUsersTodoLists(user);
    }

    @PostMapping(value = "/list/add")
    public @ResponseBody
    void createNewTodoList(@RequestBody NewTodoListRequestDto dto, Authentication authentication, Locale locale) {
        final User user = getUser(authentication, locale);

        todoService.createOrUpdateTodoList(dto, user);
    }

    @PostMapping(value = "/list/{referralCode}/add/user")
    public @ResponseBody
    void addNewUserToList(@PathVariable String referralCode, Authentication authentication, Locale locale) {
        final User user = getUser(authentication, locale);

        todoService.addNewUserToTodoList(referralCode, user);
    }

    @PostMapping(value = "/list/delete/{todoListId}")
    public @ResponseBody
    void deleteTodoList(@PathVariable Integer todoListId, Authentication authentication, Locale locale) {
        final User user = getUser(authentication, locale);

        todoService.deleteTodoList(user, todoListId);
    }

    @PostMapping(value = "/list/delete-whole/{todoListId}")
    public @ResponseBody
    void deleteWholeTodoList(@PathVariable Integer todoListId, Authentication authentication, Locale locale) {
        final User user = getUser(authentication, locale);

        todoService.deleteWholeList(user, todoListId);
    }

    @PostMapping(value = "/save")
    public @ResponseBody
    TodoItemResponseDto saveOrUpdateTodo(@RequestBody TodoItemDto todoItemDto, Authentication authentication, Locale locale) {
        final User user = getUser(authentication, locale);

        TodoItem todoItem = todoService.saveTodo(user, todoItemDto);

        return todoItem != null ? todoItem.mapTodoItemToResponse() : null;
    }

    @PostMapping(value = "/delete/{todoId}")
    public @ResponseBody
    void deleteTodo(@PathVariable Integer todoId, Authentication authentication, Locale locale) {
        final User user = getUser(authentication, locale);

        todoService.deleteTodo(user, todoId);
    }

    @PostMapping(value = "/edit")
    public @ResponseBody
    TodoItemResponseDto editTodo(@RequestBody TodoItemDto todoItemDto, Authentication authentication, Locale locale) {
        final User user = getUser(authentication, locale);

        TodoItem todoItem = todoService.editTodo(user, todoItemDto);

        return todoItem != null ? todoItem.mapTodoItemToResponse() : null;
    }

    @PostMapping(value = "/toggle/{todoId}")
    public @ResponseBody
    void toggleTodo(@PathVariable Integer todoId, Authentication authentication, Locale locale) {
        final User user = getUser(authentication, locale);

        todoService.toggleTodo(user, todoId);
    }

    @GetMapping(value = "/find")
    public @ResponseBody
    List<Integer> findTodos(@RequestParam String text, Authentication authentication, Locale locale) {
        final User user = getUser(authentication, locale);

        List<TodoItem> todosByText = todoService.findTodosByText(user, text, locale);

        return todosByText
                .stream()
                .map(TodoItem::getTodoId)
                .collect(Collectors.toList());
    }

    @PostMapping(value = "/list/{listId}/remove-user/{userId}")
    public @ResponseBody
    void removeUserFromList(@PathVariable Integer listId, @PathVariable Integer userId, Authentication authentication, Locale locale) {
        final User user = getUser(authentication, locale);

        todoService.removeUserFromList(user, userId, listId);
    }

    @PostMapping(value = "/list/{listId}/change-owner/{userId}")
    public @ResponseBody
    void changeListOwner(@PathVariable Integer listId, @PathVariable Integer userId, Authentication authentication, Locale locale) {
        final User user = getUser(authentication, locale);

        todoService.changeListOwner(user, userId, listId);
    }

    @GetMapping(value = "/list/{listId}/all-users")
    public @ResponseBody
    List<UserForListEditDto> getAllListUsers(@PathVariable Integer listId, Authentication authentication, Locale locale) {
        final User user = getUser(authentication, locale);

        return todoService.getAllListUsers(user, listId);
    }
}
