package cz.uhk.fim.smarttodo.controller;

import cz.uhk.fim.smarttodo.common.AppControllerAncestor;
import cz.uhk.fim.smarttodo.common.Constant;
import cz.uhk.fim.smarttodo.common.validation.StValidationException;
import cz.uhk.fim.smarttodo.common.validation.ValidationMessage;
import cz.uhk.fim.smarttodo.domain.User;
import cz.uhk.fim.smarttodo.dto.UserRegistrationRequestDto;
import cz.uhk.fim.smarttodo.dto.user.UserInformationDto;
import cz.uhk.fim.smarttodo.service.UserService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.core.Authentication;
import org.springframework.web.bind.annotation.*;

import java.util.Locale;

@RestController
@RequestMapping(value = "/user")
public class UserController extends AppControllerAncestor {

    private final UserService userService;

    @Autowired
    public UserController(UserService userService) {
        this.userService = userService;
    }


    @PostMapping(value = "/registration")
    public @ResponseBody
    void userRegistration(@RequestBody UserRegistrationRequestDto data) {

        userService.registerNewUser(data);
    }

    @GetMapping(value = "/info")
    public @ResponseBody
    UserInformationDto getUserInfo(@RequestParam(defaultValue = "false") boolean picture, Authentication authentication, Locale locale) {
        final User user = getUser(authentication, locale);

        return user.getUserInformation(picture);
    }

    @PostMapping(value = "/update")
    public @ResponseBody
    void userRegistration(@RequestBody UserInformationDto data, Authentication authentication, Locale locale) {
        User user = getUser(authentication, locale);
        if (!userService.isEmailUnique(data.getEmail()) && !user.getEmail().equals(data.getEmail()))
            throw new StValidationException(new ValidationMessage("email", "registration.email.not.unique", Constant.EMPTY_ARRAY));

        user.setEmail(data.getEmail());
        user.setFirstName(data.getFirstName());
        user.setLastName(data.getLastName());
        user.setAvatar(data.getAvatar());

        userService.updateUser(user);
    }
}
