package cz.uhk.fim.smarttodo.domain;

import org.apache.commons.lang3.time.DateUtils;

import javax.persistence.*;
import java.util.*;
import java.util.stream.Collectors;

@Entity
@Table(name = "\"TODO_LIST\"")
public class TodoList {

    @Id
    @GeneratedValue(strategy = GenerationType.IDENTITY)
    private Integer listId;

    @Column
    private String name;

    @Column
    private String icon;

    @Column
    private String color;

    @Column(unique = true)
    private String referralCode;

    @ManyToOne
    private User listOwner;

    @OneToMany(mappedBy = "todoList", cascade = CascadeType.REMOVE)
    @OrderBy("todoId ASC")
    private List<TodoItem> items = new ArrayList<>();

    @ManyToMany(mappedBy = "todoLists")
    private List<User> users = new ArrayList<>();

    public Integer getListId() {
        return listId;
    }

    public void setListId(Integer listId) {
        this.listId = listId;
    }

    public String getName() {
        return name;
    }

    public void setName(String name) {
        this.name = name;
    }

    public String getIcon() {
        return icon;
    }

    public void setIcon(String icon) {
        this.icon = icon;
    }

    public String getColor() {
        return color;
    }

    public void setColor(String color) {
        this.color = color;
    }

    public String getReferralCode() {
        return referralCode;
    }

    public void setReferralCode(String referralCode) {
        this.referralCode = referralCode;
    }

    public User getListOwner() {
        return listOwner;
    }

    public void setListOwner(User listOwner) {
        this.listOwner = listOwner;
    }

    public List<TodoItem> getItems() {
        return items;
    }

    public void setItems(List<TodoItem> items) {
        this.items = items;
    }

    public List<User> getUsers() {
        return users;
    }

    public void setUsers(List<User> users) {
        this.users = users;
    }

    public void clearList() {

        this.users.forEach(user -> {
            user.getTodoLists().remove(this);
        });

        this.getUsers().clear();
    }

    public void removeUserFromList(Integer userId) {
        Optional<User> userToRemove = this.users
                .stream()
                .filter(user -> user.getId().equals(userId))
                .findFirst();

        if (userToRemove.isPresent()) {
            User user = userToRemove.get();
            user.removeTodoList(this);
        }
    }

    public List<TodoItem> getTodosForDate(Date date) {
        List<TodoItem> todos = new ArrayList<>();

        this.items.forEach(t -> {
            if (t.getOnDate() != null && t.getTodoType().equals(TodoItem.TodoType.TODO) && DateUtils.isSameDay(t.getOnDate(), date)) {
                todos.add(t);
            } else {
                t.getNestedTodoItems().forEach(nt -> {
                    if (nt.getOnDate() != null && DateUtils.isSameDay(nt.getOnDate(), date))
                        todos.add(nt);
                });
            }
        });

        return todos;
    }

    public List<TodoItem> getTodosFromDate(Date from) {
        List<TodoItem> todos = new ArrayList<>();

        this.items.forEach(t -> {
            if (t.getOnDate() != null && t.getTodoType().equals(TodoItem.TodoType.TODO) && (DateUtils.isSameDay(t.getOnDate(), from) || !t.getOnDate().before(from))) {
                todos.add(t);
            } else {
                t.getNestedTodoItems().forEach(nt -> {
                    if (nt.getOnDate() != null && (DateUtils.isSameDay(nt.getOnDate(), from) || !nt.getOnDate().before(from)))
                        todos.add(nt);
                });
            }
        });

        return todos;
    }

    public List<TodoItem> getTodosToDate(Date to) {
        List<TodoItem> todos = new ArrayList<>();

        this.items.forEach(t -> {
            if (t.getOnDate() != null && t.getTodoType().equals(TodoItem.TodoType.TODO) && (DateUtils.isSameDay(t.getOnDate(), to) || !t.getOnDate().after(to))) {
                todos.add(t);
            } else {
                t.getNestedTodoItems().forEach(nt -> {
                    if (nt.getOnDate() != null && (DateUtils.isSameDay(nt.getOnDate(), to) || !nt.getOnDate().after(to)))
                        todos.add(nt);
                });
            }
        });

        return todos;
    }

    public List<TodoItem> getTodosBetweenDate(Date from, Date to) {
        List<TodoItem> todos = new ArrayList<>();

        this.items.forEach(t -> {
            if (t.getOnDate() != null && t.getTodoType().equals(TodoItem.TodoType.TODO) &&
                    (DateUtils.isSameDay(t.getOnDate(), from) || !t.getOnDate().before(from)) &&
                    (DateUtils.isSameDay(t.getOnDate(), to) || !t.getOnDate().after(to))) {
                todos.add(t);
            } else {
                t.getNestedTodoItems().forEach(nt -> {
                    if (nt.getOnDate() != null &&
                            (DateUtils.isSameDay(nt.getOnDate(), from) || !nt.getOnDate().before(from)) &&
                            (DateUtils.isSameDay(nt.getOnDate(), to) || !nt.getOnDate().after(to)))
                        todos.add(nt);
                });
            }
        });

        return todos;
    }

}

