package cz.uhk.fim.smarttodo.domain;

import cz.uhk.fim.smarttodo.dto.TodoItemResponseDto;
import cz.uhk.fim.smarttodo.dto.TodoListResponseDto;
import cz.uhk.fim.smarttodo.dto.user.UserInformationDto;
import org.apache.commons.lang3.time.DateUtils;
import org.hibernate.annotations.CreationTimestamp;
import org.springframework.security.core.GrantedAuthority;
import org.springframework.security.core.userdetails.UserDetails;

import javax.persistence.*;
import java.util.*;

@Entity
@Table(name = "\"USER\"")
public class User implements UserDetails {

    @Id
    @GeneratedValue(strategy = GenerationType.IDENTITY)
    private Integer id;

    @Column(nullable = false, unique = true)
    private String username;

    @Column(nullable = false)
    private String password;

    @Column
    private String firstName;

    @Column
    private String lastName;

    @Lob
    private String avatar;

    @Column(nullable = false)
    private String email;

    @ManyToMany(cascade = {CascadeType.ALL})
    @JoinTable(
            name = "User_Todo_List",
            joinColumns = {@JoinColumn(name = "id")},
            inverseJoinColumns = {@JoinColumn(name = "list_Id")}
    )
    @OrderBy("listId ASC")
    private List<TodoList> todoLists = new ArrayList<>();

    @OneToMany(mappedBy = "user")
    private List<Event> eventList = new ArrayList<>();

    @CreationTimestamp
    private Date createdOn;


    public Integer getId() {
        return id;
    }

    public void setId(Integer id) {
        this.id = id;
    }

    public void setUsername(String username) {
        this.username = username;
    }

    public void setPassword(String password) {
        this.password = password;
    }

    public String getEmail() {
        return email;
    }

    public void setEmail(String email) {
        this.email = email;
    }

    public String getFirstName() {
        return firstName;
    }

    public void setFirstName(String firstName) {
        this.firstName = firstName;
    }

    public String getLastName() {
        return lastName;
    }

    public void setLastName(String lastName) {
        this.lastName = lastName;
    }

    public List<TodoList> getTodoLists() {
        return todoLists;
    }

    public void setTodoLists(List<TodoList> todoLists) {
        this.todoLists = todoLists;
    }

    public String getAvatar() {
        return avatar;
    }

    public void setAvatar(String avatar) {
        this.avatar = avatar;
    }

    public List<Event> getEventList() {
        return eventList;
    }

    public void setEventList(List<Event> eventList) {
        this.eventList = eventList;
    }

    public Date getCreatedOn() {
        return createdOn;
    }

    public void setCreatedOn(Date createdOn) {
        this.createdOn = createdOn;
    }

    public List<TodoListResponseDto> getAllUsersTodoLists() {
        if (this.todoLists.isEmpty()) return new ArrayList<>();

        Date today = new Date();

        List<TodoListResponseDto> todoLists = new ArrayList<>();
        for (TodoList todoList : this.todoLists) {
            TodoListResponseDto listDto = new TodoListResponseDto();
            listDto.setListId(todoList.getListId());
            listDto.setName(todoList.getName());
            listDto.setColor(todoList.getColor());
            listDto.setIcon(todoList.getIcon());
            listDto.setReferralCode(todoList.getReferralCode());
            listDto.setGroup(todoList.getUsers().size() > 1);
            listDto.setOwner(todoList.getListOwner().getId().equals(this.id));

            int todosCount = 0;
            int allTodosCount = 0;
            int flaggedTodosCount = 0;
            int todayTodosCount = 0;
            int scheduledTodosCount = 0;

            List<TodoItemResponseDto> todoItems = new ArrayList<>();
            for (TodoItem todoItem : todoList.getItems()) {
                if (todoItem.getTodoType() != TodoItem.TodoType.TODO) continue;
                TodoItemResponseDto todoDto = new TodoItemResponseDto();
                todoDto.setTodoId(todoItem.getTodoId());
                todoDto.setName(todoItem.getName());
                todoDto.setCreatedOn(todoItem.getCreatedOn());
                todoDto.setTodoType(todoItem.getTodoType());
                todoDto.setDone(todoItem.isDone());
                todoDto.setFlagged(todoItem.isFlagged());
                todoDto.setOnDate(todoItem.getOnDate());
                todoDto.setListId(todoItem.getTodoListId());
                todoDto.setPlace(todoItem.getPlace() != null ? todoItem.getPlace().mapPlaceToResponse() : null);

                List<TodoItemResponseDto> nestedTodoItems = new ArrayList<>();
                for (TodoItem nestedTodoItem : todoItem.getNestedTodoItems()) {
                    TodoItemResponseDto nestedTodoDto = new TodoItemResponseDto();
                    nestedTodoDto.setTodoId(nestedTodoItem.getTodoId());
                    nestedTodoDto.setName(nestedTodoItem.getName());
                    nestedTodoDto.setCreatedOn(nestedTodoItem.getCreatedOn());
                    nestedTodoDto.setTodoType(nestedTodoItem.getTodoType());
                    nestedTodoDto.setNestedTodoItems(null);
                    nestedTodoDto.setDone(nestedTodoItem.isDone());
                    nestedTodoDto.setFlagged(nestedTodoItem.isFlagged());
                    nestedTodoDto.setOnDate(nestedTodoItem.getOnDate());
                    nestedTodoDto.setListId(nestedTodoItem.getTodoListId());
                    nestedTodoDto.setPlace(nestedTodoItem.getPlace() != null ? nestedTodoItem.getPlace().mapPlaceToResponse() : null);

                    nestedTodoItems.add(nestedTodoDto);

                    if (!nestedTodoItem.isDone()) {
                        todosCount++;
                        if (nestedTodoItem.isFlagged()) flaggedTodosCount++;
                        if (nestedTodoItem.getOnDate() != null) {
                            scheduledTodosCount++;
                            if (DateUtils.isSameDay(nestedTodoItem.getOnDate(), today)) todayTodosCount++;
                        }
                    }
                    allTodosCount++;
                }

                todoDto.setNestedTodoItems(nestedTodoItems);
                todoItems.add(todoDto);

                if (!todoItem.isDone()){
                    todosCount++;
                    if (todoItem.isFlagged()) flaggedTodosCount++;
                    if (todoItem.getOnDate() != null) {
                        scheduledTodosCount++;
                        if (DateUtils.isSameDay(todoItem.getOnDate(), today)) todayTodosCount++;
                    }
                }

                allTodosCount++;
            }

            listDto.setTodos(todoItems);

            listDto.setTodosCount(todosCount);
            listDto.setAllTodosCount(allTodosCount);
            listDto.setFlaggedTodosCount(flaggedTodosCount);
            listDto.setScheduledTodosCount(scheduledTodosCount);
            listDto.setTodayTodosCount(todayTodosCount);

            todoLists.add(listDto);
        }

        return todoLists;
    }

    public void removeTodoList(TodoList todoList) {
        todoLists.remove(todoList);
        todoList.getUsers().remove(this);
    }

    public UserInformationDto getUserInformation(boolean picture) {
        UserInformationDto dto = new UserInformationDto();
        dto.setFirstName(this.firstName);
        dto.setLastName(this.lastName);
        dto.setUsername(this.username);
        dto.setEmail(this.email);
        dto.setAvatar(this.avatar);
        dto.setCreatedDate(this.createdOn);

        return dto;
    }

    @Override
    public Collection<? extends GrantedAuthority> getAuthorities() {
        return new ArrayList<>();
    }

    @Override
    public String getPassword() {
        return this.password;
    }

    @Override
    public String getUsername() {
        return this.username;
    }

    @Override
    public boolean isAccountNonExpired() {
        return true;
    }

    @Override
    public boolean isAccountNonLocked() {
        return true;
    }

    @Override
    public boolean isCredentialsNonExpired() {
        return true;
    }

    @Override
    public boolean isEnabled() {
        return true;
    }

    public boolean hasAccessToTodoList(Integer listId) {

        Optional<TodoList> foundList = this.getTodoLists()
                .stream()
                .filter(list -> list.getListId().equals(listId))
                .findFirst();

        return foundList.isPresent();
    }
}
