package cz.uhk.fim.smarttodo.dto.witAi;

import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;

import com.joestelmach.natty.*;
import cz.uhk.fim.smarttodo.common.Constant;
import edu.emory.mathcs.backport.java.util.Arrays;

public class ReceivedMessageDto {

    /**
     * Whole data received from Wit.ai
     */
    private Entity entities;

    // GETTERS AND SETTERS

    public Entity getEntities() {
        return entities;
    }

    public void setEntities(Entity entities) {
        this.entities = entities;
    }

    public String getPossibleEnSentence(String text) {
        final String start = "Find todos ";

        if (entities.intent != null && entities.intent[0] != null) {
            String intent = entities.intent[0].getValue();

            if (intent.equals(Constant.TODAY) || intent.equals(Constant.TOMORROW))
                return String.format("%sfor %s", start, intent);

            if (intent.equals(Constant.ALL))
                return String.format("%s%s", start, intent);

            if (Arrays.asList(Constant.WEEK_DAYS).contains(intent))
                return String.format("%son %s", start, intent);

            switch (intent) {
                case Constant.BETWEEN:
                    String[] splitedText = {};

                    String[] splitA = text.split(" a ");
                    if (splitA.length > 1) {
                        splitedText = splitA;
                    } else {
                        String[] splitAz = text.split(" až ");
                        if (splitAz.length > 1)
                            splitedText = splitAz;
                        else {
                            String[] splitAz2 = text.split(" az ");
                            if (splitAz2.length > 1)
                                splitedText = splitAz2;
                        }
                    }

                    if (splitedText.length > 1) {
                        String from = parseStringDateFromText(splitedText[0]);
                        String to = parseStringDateFromText(splitedText[1]);
                        if (from != null && to != null)
                            return String.format("%s%s %s and %s", start, intent, from, to);
                        break;
                    }
                    String s = parseStringDateFromText(text);
                    if (s != null)
                        return String.format("%sfor %s", start, s);
                    break;
                default:
                case Constant.FROM:
                case Constant.TILL:
                    String date = parseStringDateFromText(text);
                    if (date != null)
                        return String.format("%s%s %s", start, intent, date);
                    break;
            }
        }

        String s = parseStringDateFromText(text);
        if (s != null)
            return String.format("%sfor %s", start, s);

        return "";
    }

    private String parseStringDateFromText(String text) {
        DateFormat format = new SimpleDateFormat("MM/dd/yyyy");

        List<DateGroup> parsedDates = new Parser().parse(text);
        if (parsedDates != null && parsedDates.size() > 0 && parsedDates.get(0) != null) {
            List<Date> dates = parsedDates.get(0).getDates();

            if (dates.size() > 0 && dates.get(0) != null)
                return format.format(dates.get(0));
        }
        return null;
    }

    /**
     * Main Entity which contains all parsed data
     */
    public static class Entity {
        /**
         * Array of data which are related with date
         */
        private Datetime[] datetime;
        private Intent[] intent;

        // GETTERS AND SETTERS

        public Datetime[] getDatetime() {
            return datetime;
        }

        public void setDatetime(Datetime[] datetime) {
            this.datetime = datetime;
        }

        public Intent[] getIntent() {
            return intent;
        }

        public void setIntent(Intent[] intent) {
            this.intent = intent;
        }

        /**
         * Class which contains all data related with date (e.g. today, tomorrow, 4/20/2020 ...)
         */
        public static class Datetime {
            private Value from;
            private Value to;
            private Date value;

            // GETTERS AND SETTERS

            public Value getFrom() {
                return from;
            }

            public void setFrom(Value from) {
                this.from = from;
            }

            public Value getTo() {
                return to;
            }

            public void setTo(Value to) {
                this.to = to;
            }

            public Date getValue() {
                return value;
            }

            public void setValue(Date value) {
                this.value = value;
            }

            /**
             * Class which contains value of {@link Datetime}
             */
            public static class Value {
                private Date value;

                // GETTERS AND SETTERS

                public Date getValue() {
                    return value;
                }

                public void setValue(Date value) {
                    this.value = value;
                }
            } // Datetime - Value class end
        } // Datetime class end

        /**
         * Class which contains all custom data
         */
        public static class Intent {
            private String value;

            // GETTERS AND SETTERS

            public String getValue() {
                return value;
            }

            public void setValue(String value) {
                this.value = value;
            }
        } // Intent class end
    }// Entity class end
}
