package cz.uhk.fim.smarttodo.service;

import cz.uhk.fim.smarttodo.domain.Event;
import cz.uhk.fim.smarttodo.domain.User;
import cz.uhk.fim.smarttodo.dto.event.EventDto;
import cz.uhk.fim.smarttodo.repository.EventRepository;
import cz.uhk.fim.smarttodo.repository.UserRepository;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;
import java.util.Optional;

@Service
public class EventService {

    private final EventRepository eventRepository;
    private final UserRepository userRepository;

    @Autowired
    public EventService(EventRepository eventRepository, UserRepository userRepository) {
        this.eventRepository = eventRepository;
        this.userRepository = userRepository;
    }

    public List<EventDto> getAllUsersEvents(User user){
        List<EventDto> events = new ArrayList<>();

        for (Event event : user.getEventList())
            events.add(event.mapToEventDto());

        return events;
    }

    public Event createNewEvent(User user, EventDto dto){
        Event event = new Event();
        event.setName(dto.getName());
        event.setColor(dto.getColor());
        event.setOnDate(dto.getOnDate());
        event.setUser(user);

        return eventRepository.save(event);
    }

    public void deleteEvent(User user, Integer eventId){
        Optional<Event> eventDB = eventRepository.findById(eventId);

        if (eventDB.isPresent()){
            Event event = eventDB.get();

            if (isUsersEvent(user, event)){
                eventRepository.delete(event);
            }
        }
    }

    public Event updateEvent(User user, EventDto dto){
        Optional<Event> eventDb = eventRepository.findById(dto.getEventId());

        if (eventDb.isPresent()){
            Event event = eventDb.get();

            if (isUsersEvent(user, event)) {
                event.setName(dto.getName());
                event.setColor(dto.getColor());
                event.setOnDate(dto.getOnDate());

                return eventRepository.save(event);
            }
        }
        return null;
    }

    private boolean isUsersEvent(User user, Event event) {
        return user.getEventList()
                .stream()
                .anyMatch(e -> e.getEventId().equals(event.getEventId()));
    }
}
