package cz.uhk.fim.smarttodo.service;

import cz.uhk.fim.smarttodo.common.Constant;
import cz.uhk.fim.smarttodo.common.validation.StValidationException;
import cz.uhk.fim.smarttodo.common.validation.ValidationMessage;
import cz.uhk.fim.smarttodo.domain.User;
import cz.uhk.fim.smarttodo.dto.UserRegistrationRequestDto;
import cz.uhk.fim.smarttodo.dto.user.UserInformationDto;
import cz.uhk.fim.smarttodo.repository.UserRepository;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.crypto.password.PasswordEncoder;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;

@Service
public class UserService {

    private final PasswordEncoder passwordEncoder;
    private final UserRepository userRepository;

    @Autowired
    public UserService(UserRepository userRepository, PasswordEncoder passwordEncoder) {
        this.userRepository = userRepository;
        this.passwordEncoder = passwordEncoder;
    }

    public void registerNewUser(UserRegistrationRequestDto data) {
        User user = new User();
        user.setUsername(data.getUsername());
        user.setEmail(data.getEmail());
        user.setPassword(passwordEncoder.encode(data.getPassword()));

        validNewUser(user);
        userRepository.save(user);
    }

    private void validNewUser(User user) {
        List<ValidationMessage> messages = new ArrayList<>();

        if (StringUtils.isEmpty(user.getUsername()))
            messages.add(new ValidationMessage("username", "registration.username.empty", Constant.EMPTY_ARRAY));
        else if (user.getUsername().length() < 4 || user.getUsername().length() > 15) {
            messages.add(new ValidationMessage("username", "registration.username.length", Constant.EMPTY_ARRAY));
        } else if (!isUsernameUnique(user.getUsername())) {
            messages.add(new ValidationMessage("username", "registration.username.not.unique", Constant.EMPTY_ARRAY));
        }

        if (StringUtils.isEmpty(user.getEmail()))
            messages.add(new ValidationMessage("email", "registration.email.empty", Constant.EMPTY_ARRAY));
        else if (!isEmailUnique(user.getEmail())) {
            messages.add(new ValidationMessage("email", "registration.email.not.unique", Constant.EMPTY_ARRAY));
        }

        if (StringUtils.isEmpty(user.getPassword()))
            messages.add(new ValidationMessage("email", "registration.password.empty", Constant.EMPTY_ARRAY));

        if (!messages.isEmpty())
            throw new StValidationException(messages);
    }

    public boolean isEmailUnique(String email) {
        User user = userRepository.findByEmail(email);

        return user == null;
    }

    private boolean isUsernameUnique(String username) {
        User user = userRepository.findByUsername(username);

        return user == null;
    }

    public void updateUser(User user) {
        userRepository.save(user);
    }
}
