import Vue from "vue";
import VueRouter from "vue-router";
import Home from "../views/Home.vue";
import Page404 from "../views/404.vue";

import store from "../store/index";

Vue.use(VueRouter);

const routes = [
  {
    path: "/",
    name: "home",
    meta: {
      layout: "public"
    },
    component: Home
  },
  {
    path: "/todo",
    name: "todo",
    meta: {
      requiresAuth: true
    },
    component: () => import("../views/Todo.vue")
  },
  {
    path: "/profile",
    name: "profile",
    meta: {
      requiresAuth: true
    },
    component: () => import("../views/Profile.vue")
  },
  {
    path: "/map",
    name: "map",
    meta: {
      requiresAuth: true
    },
    component: () => import("../views/Map.vue")
  },
  {
    path: "/events",
    name: "events",
    meta: {
      requiresAuth: true
    },
    component: () => import("../views/Events.vue")
  },
  {
    path: "/login",
    name: "login",
    meta: { layout: "basic" },
    component: () => import("../views/Login.vue"),
    beforeEnter: (to, from, next) => {
      if (!store.getters.isLoggedIn) {
        next();
      } else {
        next({
          name: "home"
        });
      }
    }
  },
  {
    path: "*",
    name: "404",
    meta: { layout: "basic" },
    component: Page404
  }
];

const router = new VueRouter({
  mode: "history",
  base: process.env.BASE_URL,
  routes
});

router.beforeEach((to, from, next) => {
  if (to.matched.some(record => record.meta.requiresAuth)) {
    if (store.getters.isLoggedIn) {
      next();
      return;
    }
    next({ path: "/", hash: "login" });
  } else {
    next();
  }
});

export default router;
