import Axios from "axios";
import AxiosCommon from "../../axios/axios-common";
import router from "../../router";

const state = {
  token: null,
  userInfo: null
};

const getters = {
  isLoggedIn: state => !!state.token,
  getUsername: ({ userInfo }) => {
    if (!userInfo) return " ";

    if (userInfo.firstName && userInfo.lastName)
      return `${userInfo.firstName} ${userInfo.lastName}`;

    return userInfo.username;
  }
};

const mutations = {
  AUTH_USER(state, token) {
    state.token = token;
  },
  LOGOUT_USER(state) {
    state.token = null;
    state.userInfo = null;
  },
  SAVE_USER_INFO(state, userInfo) {
    state.userInfo = userInfo;
  }
};

const actions = {
  login({ dispatch, commit }, { username, password }) {
    return new Promise((resolve, reject) => {
      Axios.post(
        "user/login",
        {
          username: username,
          password: password
        },
        {
          headers: {
            "Content-Type": "text/plain"
          }
        }
      )
        .then(({ headers }) => {
          const token = headers.authorization.replace("Bearer ", "");

          commit("AUTH_USER", token);
          localStorage.setItem("token", token);

          dispatch("getUserInfo");
          resolve();
        })
        .catch(err => {
          reject(err);
        });
    });
  },
  registerUser({ commit }, registartionData) {
    return new Promise((resolve, reject) => {
      AxiosCommon.post("user/registration", registartionData)
        .then(result => {
          resolve();
        })
        .catch(err => {
          reject(err.response.data);
        });
    });
  },
  editProfile({ commit, state }, profileData) {
    return new Promise((resolve, reject) => {
      AxiosCommon.post("user/update", profileData)
        .then(result => {
          commit("SAVE_USER_INFO", {
            ...profileData,
            username: state.userInfo.username,
            createdDate: state.userInfo.createdDate
          });
          resolve();
        })
        .catch(err => {
          reject(err.response.data);
        });
    });
  },
  logout({ commit }) {
    return new Promise((resolve, reject) => {
      commit("LOGOUT_USER");
      localStorage.removeItem("token");

      router.push("/");
      resolve();
    });
  },
  tryAutoLogin({ dispatch, commit }) {
    const token = localStorage.getItem("token");
    if (!token) return;

    commit("AUTH_USER", token);

    dispatch("getUserInfo");
  },
  getUserInfo({ commit }) {
    return new Promise((resolve, reject) => {
      AxiosCommon.get("user/info")
        .then(result => {
          commit("SAVE_USER_INFO", result.data);
          resolve();
        })
        .catch(err => {
          reject(err);
        });
    });
  }
};

export default {
  state,
  getters,
  mutations,
  actions
};
